@@ -35,6 +35,8 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
   private boolean keepAspect;
   private boolean ignoreAspect;
   private int enableKate;
+  private String enableKateLanguage = "";
+  private String enableKateCategory = "";
   private Component component;
   private int bufferSize = -1;
   private int bufferLow = -1;
@@ -213,22 +215,33 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
         Debug.debug("No Kate selector yet, creating one");
 
         /* insert an overlay before the video sink */
-        ovsinkpad.unlink();
-        videodec.getPad("src").unlink();
-        overlay = ElementFactory.makeByName("kateoverlay", "overlay");
-        if (overlay == null) {
-          noSuchElement ("overlay");
-          return;
+        if (videodec != null) {
+          ovsinkpad.unlink();
+          videodec.getPad("src").unlink();
+          overlay = ElementFactory.makeByName("kateoverlay", "overlay");
+          if (overlay == null) {
+            noSuchElement ("overlay");
+            return;
+          }
+          ovsinkpad = overlay.getPad("videosink");
+          oksinkpad = overlay.getPad("katesink");
+          if (!videodec.getPad("src").link(ovsinkpad)) {
+            postMessage (Message.newError (this, "Failed linking video decoder to overlay"));
+            return;
+          }
+          add(overlay);
+          overlay.setProperty ("component", component);
+          overlay.getPad("videosrc").link(videosink.getPad("sink"));
         }
-        ovsinkpad = overlay.getPad("videosink");
-        oksinkpad = overlay.getPad("katesink");
-        if (!videodec.getPad("src").link(ovsinkpad)) {
-          postMessage (Message.newError (this, "Failed linking video decoder to overlay"));
-          return;
+        else {
+          Element fakesink = ElementFactory.makeByName("fakesink", "fakesink");
+          if (fakesink == null) {
+            noSuchElement("fakesink");
+            return;
+          }
+          oksinkpad = fakesink.getPad("sink");
+          add(fakesink);
         }
-        add(overlay);
-        overlay.setProperty ("component", component);
-        overlay.getPad("videosrc").link(videosink.getPad("sink"));
 
         kselector = ElementFactory.makeByName("selector", "selector");
         if (kselector == null) {
@@ -236,11 +249,15 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
           return;
         }
         add(kselector);
+
         if (!kselector.getPad("src").link(oksinkpad)) {
           postMessage (Message.newError (this, "Failed linking Kate selector to overlay"));
           return;
         }
+
         kselector.setState (PAUSE);
+
+	((Cortado)component).status.setHaveSubtitles(true);
       }
       tmp_katesink = kselector;
 
@@ -277,6 +294,15 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
       if (enableKate == katedec.size()-1) {
         doEnableKateIndex(enableKate);
       }
+      else if (enableKate < 0 && (!enableKateLanguage.equals("") || !enableKateCategory.equals(""))) {
+        String language = caps.getFieldString("language", "");
+        String category = caps.getFieldString("category", "");
+        boolean matching_language = enableKateLanguage.equals("") || enableKateLanguage.equals(language);
+        boolean matching_category = enableKateCategory.equals("") || enableKateCategory.equals(category);
+        if (matching_language && matching_category) {
+          doEnableKateIndex(katedec.size()-1);
+        }
+      }
     }
   }
   
@@ -307,6 +333,9 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
       remove (audiosink);
       audiosink = null;
       changed = true;
+      if (videosink != null) {
+        videosink.setProperty ("max-lateness", Long.toString(Long.MAX_VALUE));
+      }
     }
     if (vpad == null && enableVideo) {
       Debug.log(Debug.INFO, "file has no video, remove videosink");
@@ -314,23 +343,9 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
       if (overlay != null) {
         overlay.setState(STOP);
       }
-      for (int n=0; n<katedec.size(); ++n) {
-        el = (Element)katedec.elementAt(n);
-        el.setState(STOP);
-        remove(el);
-        el = (Element)k_queue.elementAt(n);
-        el.setState(STOP);
-        remove(el);
-      }
-      if (kselector != null) {
-        kselector.setState(STOP);
-        remove(kselector);
-        kselector = null;
-      }
+
       remove (videosink);
       remove (overlay);
-      katedec.removeAllElements();
-      k_queue.removeAllElements();
       videosink = null;
       overlay = null;
       changed = true;
@@ -413,7 +428,11 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
    * A negative index and empty language/category strings will enable none.
    */
   public void enableKateStream(int idx, String language, String category) {
-    if (idx < 0) idx = findKateStream(language, category);
+    if (idx < 0) {
+      enableKateLanguage = language;
+      enableKateCategory = category;
+      idx = findKateStream(language, category);
+    }
     if (idx == enableKate) return;
     doEnableKateIndex(idx);
   }
@@ -627,7 +646,8 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
       videosink.setProperty ("component", component);
       resize(component.getSize());
 
-      videosink.setProperty ("max-lateness", Long.toString(Clock.MSECOND * 20));
+      videosink.setProperty ("max-lateness", 
+         Long.toString(enableAudio ? Clock.MSECOND * 20 : Long.MAX_VALUE));
       add(videosink);
 
       ovsinkpad = videosink.getPad("sink");
@@ -650,11 +670,20 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
       Debug.log(Debug.INFO, "using high quality javax.sound backend");
     }
     catch (Throwable e) {
-      s = (com.fluendo.plugin.AudioSink)ElementFactory.makeByName("audiosinksa", "audiosink");
-      Debug.log(Debug.INFO, "using low quality sun.audio backend");
+      try {
+        Class.forName("sun.audio.AudioStream");
+        Class.forName("sun.audio.AudioPlayer");
+        s = (com.fluendo.plugin.AudioSink)ElementFactory.makeByName("audiosinksa", "audiosink");
+        Debug.log(Debug.INFO, "using low quality sun.audio backend");
+      }
+      catch (Throwable e2) {
+        s = null;
+        Debug.log(Debug.INFO, "No audio backend available");
+      }
     }
     if (s == null) {
-      noSuchElement ("audiosink");
+      Debug.warn ("Failed to create an audio sink, continuing anyway");
+      //noSuchElement ("audiosink");
       return null;
     }
     if (!s.test()) {
@@ -788,4 +817,16 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
     }
     return result;
   }
+
+  protected int getNumKateStreams() {
+    return katedec.size();
+  }
+  protected String getKateStreamCategory(int idx) {
+    if (idx < 0 || idx >= katedec.size()) return "";
+    return String.valueOf(((Element)katedec.elementAt(idx)).getProperty("category"));
+  }
+  protected String getKateStreamLanguage(int idx) {
+    if (idx < 0 || idx >= katedec.size()) return "";
+    return String.valueOf(((Element)katedec.elementAt(idx)).getProperty("language"));
+  }
 }
