@@ -21,6 +21,7 @@ package com.fluendo.player;
 import java.awt.*;
 import java.awt.image.*;
 import java.awt.event.*;
+import java.awt.font.*;
 import java.util.*;
 
 public class Status extends Component implements MouseListener,
@@ -37,24 +38,29 @@ public class Status extends Component implements MouseListener,
     private Component component;
 
     private Font font = new Font("SansSerif", Font.PLAIN, 10);
+    private Font boldFont = null;
 
     private boolean haveAudio;
+    private boolean haveSubtitles;
     private boolean havePercent;
     private boolean seekable;
     private boolean live;
     private boolean showSpeaker;
+    private boolean showSubtitles;
     private boolean clearedScreen;
 
-    private static final int NONE = 0;
-    private static final int BUTTON1 = 1;
-    private static final int BUTTON2 = 2;
-    private static final int SEEKER = 3;
-    private static final int SEEKBAR = 4;
+    private static final int NONE = -1;
+    private static final int BUTTON1 = 0;
+    private static final int BUTTON2 = 1;
+    private static final int SEEKER = 2;
+    private static final int SEEKBAR = 3;
+    private static final int AUDIO = 4;
+    private static final int SUBTITLES = 5;
     private int clicked = NONE;
 
-    private Color button1Color;
-    private Color button2Color;
-    private Color seekColor;
+    private Color colors[] = {
+      Color.black, Color.black, Color.black, Color.black, Color.black, Color.black
+    };
 
     private static final int SPEAKER_WIDTH = 12;
     private static final int SPEAKER_HEIGHT = 10;
@@ -85,22 +91,26 @@ public class Status extends Component implements MouseListener,
     private Image speakerImg;
     private int speakerWidth; // width of the speaker icon or zero if hidden
 
+    private int subtitlesWidth; // width of the subtitles icon or zero if hidden
+
 
     private Vector listeners = new Vector();
 
+    public Image createImage(Component comp, String s, int w, int h) {
+        int[] pixels = new int[w * h];
+
+        for (int i = 0; i < w * h; i++) {
+            pixels[i] = 0xff000000 | (s.charAt(i) << 16)
+                    | (s.charAt(i) << 8) | (s.charAt(i));
+        }
+        return comp.getToolkit().createImage(
+                new MemoryImageSource(w, h, pixels, 0, w));
+    }
+
     public Status(Component comp) {
-        int[] pixels = new int[SPEAKER_WIDTH * SPEAKER_HEIGHT];
         component = comp;
 
-        for (int i = 0; i < SPEAKER_WIDTH * SPEAKER_HEIGHT; i++) {
-            pixels[i] = 0xff000000 | (speaker.charAt(i) << 16)
-                    | (speaker.charAt(i) << 8) | (speaker.charAt(i));
-        }
-        speakerImg = comp.getToolkit().createImage(
-                new MemoryImageSource(SPEAKER_WIDTH, SPEAKER_HEIGHT, pixels, 0, SPEAKER_WIDTH));
-        button1Color = Color.black;
-        button2Color = Color.black;
-        seekColor = Color.black;
+        speakerImg = createImage(comp, speaker, SPEAKER_WIDTH, SPEAKER_HEIGHT);
     }
 
     public void addStatusListener(StatusListener l) {
@@ -113,13 +123,25 @@ public class Status extends Component implements MouseListener,
 
     public void notifyNewState(int newState) {
         for (Enumeration e = listeners.elements(); e.hasMoreElements();) {
-            ((StatusListener) e.nextElement()).newState(newState);
+            ((StatusListener) e.nextElement()).onState(newState);
         }
     }
 
     public void notifySeek(double position) {
         for (Enumeration e = listeners.elements(); e.hasMoreElements();) {
-            ((StatusListener) e.nextElement()).newSeek(position);
+            ((StatusListener) e.nextElement()).onSeek(position);
+        }
+    }
+
+    public void notifyAudio() {
+        for (Enumeration e = listeners.elements(); e.hasMoreElements();) {
+            ((StatusListener) e.nextElement()).onAudio();
+        }
+    }
+
+    public void notifySubtitles(int x, int y) {
+        for (Enumeration e = listeners.elements(); e.hasMoreElements();) {
+            ((StatusListener) e.nextElement()).onSubtitles(x, y);
         }
     }
 
@@ -137,7 +159,8 @@ public class Status extends Component implements MouseListener,
     private void paintPercent(Graphics g) {
         if (havePercent) {
             g.setColor(Color.white);
-            g.drawString("" + bufferPercent + "%", r.width - 26 - speakerWidth, r.height - 2);
+            g.drawString("" + bufferPercent + "%",
+                r.width - 26 - speakerWidth -subtitlesWidth, r.height - 2);
         }
     }
 
@@ -150,7 +173,7 @@ public class Status extends Component implements MouseListener,
 	h = r.height-2;
         g.setColor(Color.darkGray);
         g.drawRect(x, y, w, h);
-        g.setColor(button1Color);
+        g.setColor(colors[BUTTON1]);
         g.fillRect(x+1, y+1, w-1, h-1);
 
         if (state == STATE_PLAYING) {
@@ -182,7 +205,7 @@ public class Status extends Component implements MouseListener,
 
         g.setColor(Color.darkGray);
         g.drawRect(x, y, w, h);
-        g.setColor(button2Color);
+        g.setColor(colors[BUTTON2]);
         g.fillRect(x+1, y+1, w-1, h-1);
         g.setColor(Color.white);
         g.fillRect(r.height + (int)(w * .4), (int)(w * .4), (int)(w * .5), (int)(w * .5));
@@ -205,7 +228,7 @@ public class Status extends Component implements MouseListener,
      */
     private Rectangle getSeekBarRect() {
       return new Rectangle(r.height*2 + 1, 2,
-	  r.width - SEEK_TIME_GAP - TIME_WIDTH - speakerWidth - (r.height * 2), 
+	  r.width - SEEK_TIME_GAP - TIME_WIDTH - speakerWidth - subtitlesWidth - (r.height * 2), 
 	  r.height - 4);
     }
 
@@ -239,7 +262,7 @@ public class Status extends Component implements MouseListener,
         g.drawLine(tr.x + tr.width, tr.y + 1,         tr.x + tr.width,     tr.y + tr.height - 1); // Right
 
         // Thumb interior
-        g.setColor(seekColor);
+        g.setColor(colors[SEEKER]);
         g.fillRect(tr.x + 1, tr.y + 1, tr.width - 1, tr.height - 1);
     }
 
@@ -257,7 +280,7 @@ public class Status extends Component implements MouseListener,
 
         r = getBounds();
 
-        end = r.width - speakerWidth - TIME_WIDTH;
+        end = r.width - speakerWidth - subtitlesWidth - TIME_WIDTH;
 
         g.setColor(Color.white);
         g.drawString("" + hour + ":" + (min < 10 ? "0" + min : "" + min) + ":"
@@ -266,7 +289,41 @@ public class Status extends Component implements MouseListener,
 
     private void paintSpeaker(Graphics g) {
         if (haveAudio) {
-            g.drawImage(speakerImg, r.width - SPEAKER_WIDTH, r.height - SPEAKER_HEIGHT - 1, null);
+            g.drawImage(speakerImg, r.width - SPEAKER_WIDTH - subtitlesWidth, r.height - SPEAKER_HEIGHT - 1, null);
+        }
+    }
+
+    private Rectangle getSubtitlesBounds() {
+        int x = r.width - subtitlesWidth + 1;
+        int y = 1;
+	int w = r.height * 3 / 2 - 2;
+	int h = r.height - 2;
+        return new Rectangle(x, y, w, h);
+    }
+
+    private void paintSubtitles(Graphics g) {
+        if (haveSubtitles) {
+            Rectangle sb = getSubtitlesBounds();
+            int font_height = r.height - 2;
+
+            g.setColor(Color.darkGray);
+            g.drawRect(sb.x, sb.y, sb.width, sb.height);
+            g.setColor(colors[SUBTITLES]);
+            g.fillRect(sb.x+1, sb.y+1, sb.width-1, sb.height-1);
+
+            if (boldFont == null) boldFont = new Font("SansSerif", Font.BOLD, font_height);
+            g.setColor(Color.white);
+            Font previousFont = g.getFont();
+            g.setFont(boldFont);
+
+            FontMetrics fm = g.getFontMetrics();
+            float cc_w = fm.stringWidth("CC");
+            float cc_h = fm.getAscent()-fm.getDescent();
+            float button_midx = sb.x + sb.width/2.0f;
+            float button_midy = sb.y + sb.height/2.0f;
+            g.drawString("CC", (int)(button_midx - cc_w/2.0f + 0.5f), (int)(button_midy + cc_h/2.0f + 0.5f));
+
+            g.setFont(previousFont);
         }
     }
 
@@ -320,6 +377,9 @@ public class Status extends Component implements MouseListener,
         if (showSpeaker) {
             paintSpeaker(g2);
         }
+        if (showSubtitles) {
+            paintSubtitles(g2);
+        }
 
         g.drawImage(img, r.x, r.y, null);
         img.flush();
@@ -392,6 +452,12 @@ public class Status extends Component implements MouseListener,
         component.repaint();
     }
 
+    public void setHaveSubtitles(boolean a) {
+        haveSubtitles = a;
+        subtitlesWidth = showSubtitles && haveSubtitles ? r.height * 3 / 2 : 0;
+        component.repaint();
+    }
+
     public void setHavePercent(boolean p) {
         havePercent = p;
         component.repaint();
@@ -413,6 +479,12 @@ public class Status extends Component implements MouseListener,
         component.repaint();
     }
 
+    public void setShowSubtitles(boolean s) {
+        showSubtitles = s;
+        subtitlesWidth = showSubtitles && haveSubtitles ? r.height * 3 / 2 : 0;
+        component.repaint();
+    }
+
     public void setState(int aState) {
 	if (state != aState) {
           state = aState;
@@ -434,6 +506,19 @@ public class Status extends Component implements MouseListener,
         return (e.getX() >= r.height && e.getX() <= r.height + r.height-2 && e.getY() > 0 && e.getY() <= r.height-2);
     }
 
+    private boolean intersectAudio(MouseEvent e) {
+      // TODO
+      return false;
+    }
+
+    private boolean intersectSubtitles(MouseEvent e) {
+        if (r == null)
+            return false;
+
+      Rectangle bounds = getSubtitlesBounds();
+      return (e.getX() >= bounds.x && e.getX() <= bounds.x+bounds.width-2 && e.getY() > 0 && e.getY() <= bounds.height-2);
+    }
+
     private boolean intersectSeeker(MouseEvent e) {
         r = getBounds();
 	Rectangle tr = getThumbRect();
@@ -451,6 +536,10 @@ public class Status extends Component implements MouseListener,
             return BUTTON1;
         else if (intersectButton2(e))
             return BUTTON2;
+        else if (showSpeaker && haveAudio && intersectAudio(e))
+            return AUDIO;
+        else if (showSubtitles && haveSubtitles && intersectSubtitles(e))
+            return SUBTITLES;
         else if (seekable && intersectSeeker(e))
             return SEEKER;
         else if (seekable && intersectSeekbar(e))
@@ -460,9 +549,9 @@ public class Status extends Component implements MouseListener,
     }
 
     public void cancelMouseOperation() {
-      button1Color = Color.black;
-      button2Color = Color.black;
-      seekColor = Color.black;
+      for (int n=0; n<colors.length; ++n) {
+        colors[n] = Color.black;
+      }
       clicked = NONE;
     }
 
@@ -480,7 +569,7 @@ public class Status extends Component implements MouseListener,
         clicked = findComponent(e);
         if (clicked == SEEKBAR && state != STATE_STOPPED) {
           clicked = SEEKER;
-          seekColor = Color.gray;
+          colors[SEEKER] = Color.gray;
 	  mouseDragged (e);
         }
     }
@@ -521,6 +610,12 @@ public class Status extends Component implements MouseListener,
             break;
         case SEEKBAR:
             break;
+        case AUDIO:
+            notifyAudio();
+            break;
+        case SUBTITLES:
+            notifySubtitles(e.getX(), e.getY());
+            break;
         case NONE:
             break;
         }
@@ -553,48 +648,41 @@ public class Status extends Component implements MouseListener,
         }
     }
 
+    private boolean testIntersection(boolean in, int idx) {
+      if (in) {
+        if (colors[idx] != Color.gray) {
+          colors[idx] = Color.gray;
+          return true;
+        }
+      }
+      else {
+        if (colors[idx] != Color.black) {
+          colors[idx] = Color.black;
+          return true;
+        }
+      }
+      return false;
+    }
+
     public void mouseMoved(MouseEvent e) {
         boolean needRepaint = false;
 
         e.translatePoint(-1, -1);
 
 	if (!buffering) {
-            if (intersectButton1(e)) {
-	        if (button1Color != Color.gray) {
-                    button1Color = Color.gray;
-	            needRepaint = true;
-                }
-            } else {
-	        if (button1Color != Color.black) {
-                    button1Color = Color.black;
-	            needRepaint = true;
-	        }
-	    }
-	}
-        if (intersectButton2(e)) {
-            if (button2Color != Color.gray) {
-                button2Color = Color.gray;
-                needRepaint = true;
-            }
-        } else {
-	    if (button2Color != Color.black) {
-                button2Color = Color.black;
-	        needRepaint = true;
-	    }
+            if (testIntersection(intersectButton1(e), BUTTON1)) needRepaint = true;
 	}
 
+        if (testIntersection(intersectButton2(e), BUTTON2)) needRepaint = true;
+
         if (seekable) {
-            if (intersectSeeker(e)) {
-                if (seekColor != Color.gray) {
-                    seekColor = Color.gray;
-                    needRepaint = true;
-		}
-            } else {
-                if (seekColor != Color.black) {
-                    seekColor = Color.black;
-                    needRepaint = true;
-                }
-            }
+            if (testIntersection(intersectSeeker(e), SEEKER)) needRepaint = true;
+        }
+        if (haveAudio && showSpeaker) {
+            if (testIntersection(intersectAudio(e), AUDIO)) needRepaint = true;
+        }
+        if (haveSubtitles && showSubtitles) {
+            if (testIntersection(intersectSubtitles(e), SUBTITLES)) needRepaint = true;
         }
 	if (needRepaint)
           component.repaint();
