@@ -21,7 +21,7 @@ package com.fluendo.plugin;
 import java.util.*;
 import com.jcraft.jogg.*;
 
-interface OggPayload
+public interface OggPayload
 {
   /**
    * Check if the packet contains the signature
@@ -53,6 +53,10 @@ interface OggPayload
    * Get mime type
    */
   public String getMime ();
+  /**
+   * Get mime type from the given packet
+   */
+  public String getMime (Packet op);
   /**
    * Check if the stream is discontinuous (eg, no need to wait
    * for data on this stream before playing)
